# UNCERTAIN METADATA KEYS

DEFAULT_U_NAME_KEY = 'u:concept:name'
DEFAULT_U_TIMESTAMP_MIN_KEY = 'u:time:timestamp_min'
DEFAULT_U_TIMESTAMP_MAX_KEY = 'u:time:timestamp_max'
DEFAULT_U_MISSING_KEY = 'u:missing'
